import json

def convert(filename):
	i = open(filename, 'r')
	raw = i.read()
	i.close()

	name = filename[:-len('.json')]

	root = json.loads(raw)
	height = root['height']
	width = root['width']
	map = root['layers'][0]['data']
	map_offset = root['tilesets'][0]['firstgid']
	palmap = root['layers'][1]['data']
	palmap_offset = root['tilesets'][1]['firstgid']

	#Parse objmap
	bumpers = []
	spawnX = 5
	spawnY = 28
	try:
		objmap = root['layers'][2]['data']
		objmap_offset = root['tilesets'][2]['firstgid']
		for x in range(width):
			for y in range(height):
				b = objmap[y*width + x] - objmap_offset
				if(b == 1): #SpawnPoint
					spawnX = x
					spawnY = y
				if(b == 2): #Bumper
					bumpers.append((x*8, y*8))
	except:
		pass

	#for bumper in bumpers:
	#	 x, y = bumper

	h = open(name+'.h', 'w')
	h.write('struct Tile;\n')
	h.write('struct RawVector2i;\n')
	h.write('extern const Tile %s[];\n' % (name))
	h.write('extern const RawVector2i %s_bumpers[];\n' % (name))
	h.write('#define %s_WIDTH %i\n' % (str.upper(name), width))
	h.write('#define %s_HEIGHT %i\n' % (str.upper(name), height))
	h.write('#define %s_BUMPERS %i\n' % (str.upper(name), len(bumpers)))
	h.write('#define %s_SPAWN_X %i\n' % (str.upper(name), spawnX))
	h.write('#define %s_SPAWN_Y %i\n' % (str.upper(name), spawnY))
	h.close()

	c = open(name+'.cpp', 'w')
	c.write('#include "%s.h"\n' % (name))
	c.write('#include "../src/level.hpp"\n\n')
	c.write('#include "../src/util.hpp"\n\n')

	#Tilemap
	acidTiles = [10, 11, 42, 43]
	spikeTiles = [14, 15, 16, 17, 48, 49]
	blockTiles = list(range(2, 10)) + list(range(34, 42))
	waterTiles = [12, 13, 44, 45]
	checkPointTiles = [66, 98, 130, 162]
	colorWallTiles = [67, 68, 99, 100, 131, 132, 163, 164]
	c.write('const Tile %s[] = \n{' % (name))
	for i in range(len(map)):
		if i%width == 0:
			c.write('\n\t')
		tile = max(0, map[i] - map_offset)
		pal = max(0, palmap[i] - palmap_offset)
		solid = tile in blockTiles
		lethal = (tile in spikeTiles) or (tile in waterTiles) or (tile in acidTiles)
		checkpoint = (tile in checkPointTiles)
		type = solid + (lethal << 1) + (checkpoint << 2)
		if tile in colorWallTiles and pal != 0:
			type = type + ((1 << pal) << 2)
		c.write('{0x%02x, 0x%04x}, ' % (type, tile + (pal << 12)))
	c.write('\n};\n')

	#Bumpers
	c.write('\nconst RawVector2i %s_bumpers[] = \n{' % name)
	for bumper in bumpers:
		(x, y) = bumper
		c.write('\n{%i, %i},' % (x, y))
	c.write('\n};\n')

	c.write('\n\n\n\n')

	c.close()

if __name__ == "__main__":
	import sys
	if len(sys.argv) == 2:
		convert(sys.argv[1])
	else:
		print('usage : %s filename' % argv[0])





