#include "main.h"
#include "graphic.h"
#include "avatar.hpp"
#include "ninja.hpp"
#include "sound.h"
#include "fx.h"

// Note : 4 is the maximum speed
s8 jump_dy[] = {0, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -3, -3, -3, -3, -2, -2, -2, -2, -2, -1, -1, -1, -1, 0, 0};
#define FALL_START (-sizeof(jump_dy) + 1)

Avatar::Avatar() : Entity()
{
#ifdef SENTINELS
	__instanceBegin[0] = 'A';
	__instanceBegin[1] = 'v';
	__instanceBegin[2] = 't';
	__instanceBegin[3] = '{';
	__instanceEnd[0] = '}';
	__instanceEnd[1] = 'A';
	__instanceEnd[2] = 'v';
	__instanceEnd[3] = 't';
#endif // SENTINELS
}

void Avatar::init(u8 num, u8 width, u8 height)
{
	Entity::init(num, width, height);
	jumpIndex = 0;

	// blocks and color walls having a different color index
	collisionMask = Tile::BLOCK | (~(1 << index) << 3);
}

void Avatar::update()
{
	if (!active)
	{
		return;
	}

	Vector2i motion;

	//
	// Controls
	//

	// Copy controls before doing stuff with it

	if (dir < 0)
		dir = -1;
	else
		dir = 1;

	bool jumped = false;
	if (index == game.current_avatar)
	{
		if(hold_input & BUTTONRIGHT)
		{
			++motion.x;
			dir = 2;
		}
		if(hold_input & BUTTONLEFT)
		{
			--motion.x;
			dir = -2;
		}
		if(jumpIndex == 0 && (frame_input & BUTTONA))
		{
			// Jump
			if(index == 0)
				jumpIndex = jumpStart();
			else if(index == 1)
				jumpIndex = jumpStart();
			else if(index == 2)
				jumpIndex = jumpStart();
			else // 3
				jumpIndex = jumpStart();
			jumped = true;
		}
		if(hold_input & BUTTONSELECT)
		{
			game.framesBeforeRespawn = 2;
		}
	}

	//
	// Gravity
	//

	if (jumpIndex == 0)
	{
		if(!isGroundBelow())
			jumpIndex = FALL_START;
	}

	if(jumpIndex > 0)
	{
		motion.y += jump_dy[jumpIndex];
		if (jumpIndex == sizeof(jump_dy) - 1)
			jumpIndex = -jumpIndex;
		else
			++jumpIndex;
	}
	else if (jumpIndex < 0)
	{
		motion.y -= jump_dy[-jumpIndex];
		if (jumpIndex < -1)
			++jumpIndex;
	}

	//
	// Physics
	//

	// If we are moving
	if(motion.x != 0 || motion.y != 0)
	{
		Vector2i finalMotion = applyPhysics(motion);

		// If the Y motion has been nulled
		if(finalMotion.y == 0)
		{
			if(motion.y > 0)
			{
				jumpIndex = 0; // Land

				u32 bumperIndex;
				if(isBumperBelow(&bumperIndex))
				{
					animate_bumper(bumperIndex);
					jumpIndex = bumperJumpStart();
					sfxBump();
				}
			}
			else if(motion.y < 0)
			{
				jumpIndex = FALL_START; // Bunk on ceiling
			}
		}
		else if(finalMotion.y < 0)
		{
			if(jumped)
			{
				u8 sz = (index == 0 ? 0 : (index == 3 ? 2 : 1));
				__debug = index;
				sfxJump(sz);
			}
		}

		box.pos += finalMotion;

		// Checkpoints
		Vector2i tpos;
		if(game.level.pixelCollision(box, Tile::CHECKPOINT, &tpos))
		{
			game.lastCheckPoint = tpos;
			--tpos.y;
			if(game.level.tile(tpos).type & Tile::CHECKPOINT)
			{
				game.lastCheckPoint = tpos;
				--tpos.y;
				if(game.level.tile(tpos).type & Tile::CHECKPOINT)
				{
					game.lastCheckPoint = tpos;
					--tpos.y;
					if(game.level.tile(tpos).type & Tile::CHECKPOINT)
						game.lastCheckPoint = tpos;
				}
			}
		}

		// Lethal tiles :
		// Only the controlled avatar can die
		if(index == game.current_avatar)
		{
			if(game.level.pixelCollision(box, Tile::LETHAL))
				kill();
		}
	}
}

bool Avatar::isBumperBelow(u32 * bumperIndex)
{
	Rect offsetBox = box;
	++offsetBox.pos.y;
	return game.level.isBumperAtPixels(offsetBox, bumperIndex);
}

void Avatar::kill()
{
	//game.respawn();
	game.framesBeforeRespawn = 30; // Deferred respawn
	sfxDie();
	// TODO trigger some kill feedback
}

const s8 jump_start[] = {15, 17, 19, 21};
const s8 bumper_jump_start[] = {27, 23, 8, 1};
const s8 pop_jump_start[] = {23, 25, 27, 28};
