#include "main.h"
#include "entity.hpp"
#include "ninja.hpp"

Entity::Entity()
{
#ifdef SENTINELS
	__entityInstanceBegin[0] = 'E';
	__entityInstanceBegin[1] = 'n';
	__entityInstanceBegin[2] = 't';
	__entityInstanceBegin[3] = '{';
	__entityInstanceEnd[0] = '}';
	__entityInstanceEnd[1] = 'E';
	__entityInstanceEnd[2] = 'n';
	__entityInstanceEnd[3] = 't';
	__debug = 0;
#endif // SENTINELS
}

void Entity::init(u8 num, u8 width, u8 height)
{
	box.setSize(width, height);
	index = num;
	dir = 1;
	active = false;
	collisionMask = Tile::BLOCK;
}

bool Entity::isGroundBelow()
{
	Rect offsetBox = box;
	++offsetBox.pos.y;
	return game.pixelCollision(offsetBox, this);
}

Vector2i Entity::applyPhysics(Vector2i motion)
{
	Rect motionBox = box;
	Vector2i newMotion;

	if(motion.x)
	{
		motionBox.pos.x = motionBox.pos.x + (motion.x > 0 ? 1 : -1);
		if(game.pixelCollision(motionBox, this, collisionMask))
		{
			motion.x = 0;
		}
	}
	if(motion.y)
	{
		motionBox.pos.x = box.pos.x;
		motionBox.pos.y = motionBox.pos.y + (motion.y > 0 ? 1 : -1);
		if(game.pixelCollision(motionBox, this, collisionMask))
		{
			motion.y = 0;
		}
	}

	motionBox.pos = box.pos;

	s32 d;
	for(d = 4; d > 0; --d) // 4 is a maximal speed (must be a power of 2)
	{
		newMotion.x = (d*motion.x)/4;
		newMotion.y = (d*motion.y)/4;
		motionBox.pos.x = box.pos.x + newMotion.x;
		motionBox.pos.y = box.pos.y + newMotion.y;
		if(!game.pixelCollision(motionBox, this, collisionMask))
			break;
	}

	if(d == 0)
		return Vector2i();
	return newMotion;
}

