#ifndef GBA_H
#define GBA_H

/**
 * Types define
 */

typedef signed char 	s8;
typedef signed short 	s16;
typedef signed int 		s32;

typedef unsigned char 	u8;
typedef unsigned short 	u16;
typedef unsigned int 	u32;

/**
 * Platform stuff
 */

#define RESOLUTION_X 240
#define RESOLUTION_Y 160


#define REG_DISPCNT (u16 *)				0x04000000
#define REG_DISPSTAT (volatile u8 *)	0x04000004
#define REG_VCOUNT (volatile u8 *)		0x04000006

#define REG_BG0CNT (u16 *)				0x04000008
#define REG_BG1CNT (u16 *)				0x0400000A
#define REG_BG2CNT (u16 *)				0x0400000C
#define REG_BG3CNT (u16 *)				0x0400000E

#define REG_BG0HOFS (u16 *)				0x04000010
#define REG_BG0VOFS (u16 *)				0x04000012
#define REG_BG1HOFS (u16 *)				0x04000014
#define REG_BG1VOFS (u16 *)				0x04000016
#define REG_BG2HOFS (u16 *)				0x04000018
#define REG_BG2VOFS (u16 *)				0x0400001A
#define REG_BG3HOFS (u16 *)				0x0400001C
#define REG_BG3VOFS (u16 *)				0x0400001E

#define REG_WIN0H_RIGHT (u8 *)			0x04000040
#define REG_WIN0H_LEFT (u8 *)			0x04000041
#define REG_WIN1H_RIGHT (u8 *)			0x04000042
#define REG_WIN1H_LEFT (u8 *)			0x04000043
#define REG_WIN0V_BOT (u8 *)			0x04000044
#define REG_WIN0V_TOP (u8 *)			0x04000045
#define REG_WIN1V_BOT (u8 *)			0x04000046
#define REG_WIN1V_TOP (u8 *)			0x04000047
#define REG_WININ (u8 *)				0x04000048
#define REG_WINOUT (u8 *)				0x0400004A

#define REG_MOSAIC (u16 *)				0x0400004C

#define REG_BLDMOD (u16 *)				0x04000050
#define REG_COLEVA (u8 *)				0x04000052
#define REG_COLEVB (u8 *)				0x04000053
#define REG_COLEY (u16 *)				0x04000054

#define REG_SOUNDCNT_L (u16 *)			0x04000080
#define REG_SOUNDCNT_H (u16 *)			0x04000082
#define REG_SOUNDCNT_X (u16 *)			0x04000084

#define REG_SOUND1CNT_L (u16 *)			0x04000060
#define REG_SOUND1CNT_H (u16 *)			0x04000062
#define REG_SOUND1CNT_X (u16 *)			0x04000064

#define REG_SND_FIFO_A (u32 *)			0x040000A0
#define REG_SND_FIFO_B (u32 *)			0x040000A4

#define REG_DMA0SAD (vconst void **)	0x040000B0
#define REG_DMA0DAD (void **)			0x040000B4
#define REG_DMA0CNT (u32 *)				0x040000B8
#define REG_DMA1SAD (const void **)		0x040000BC
#define REG_DMA1DAD (void **)			0x040000C0
#define REG_DMA1CNT (u32 *)				0x040000C4
#define REG_DMA2SAD (const void **)		0x040000C8
#define REG_DMA2DAD (void **)			0x040000CC
#define REG_DMA2CNT (u32 *)				0x040000D0
#define REG_DMA3SAD (const void **)		0x040000D4
#define REG_DMA3DAD (void **)			0x040000D8
#define REG_DMA3CNT (u32 *)				0x040000DC

#define REG_TIMER0 (u16 *)				0x04000100
#define REG_TM0_CNT (u16 *)				0x04000102
#define REG_TIMER1 (u16 *)				0x04000104
#define REG_TM1_CNT (u16 *)				0x04000106
#define REG_TIMER2 (u16 *)				0x04000108
#define REG_TM2_CNT (u16 *)				0x0400010A
#define REG_TIMER3 (u16 *)				0x0400010C
#define REG_TM3_CNT (u16 *)				0x0400010E

#define REG_KEYINPUT (volatile u16 *)	0x04000130
#define REG_KEYCNT (u16 *) 				0x04000132

#define REG_IE (u16 *) 					0x04000200
#define REG_IF (volatile u16 *) 		0x04000202
#define REG_IME (u16*) 					0x04000208

#define REG_IRQPTR (void (**)())		0x03007FFC

enum DISPCNT
{
	MODE_0			= 0x0000,
	MODE_1			= 0x0001,
	MODE_2			= 0x0002,
	MODE_3			= 0x0003,
	MODE_4			= 0x0004,
	MODE_5			= 0x0005,
	DISPLAY_FRAME 	= 0x0010,
	H_BLANK_OBJ		= 0x0020,
	OBJ_MAP_1D		= 0x0040,
	FORCE_VBLANK	= 0x0080,
	BG0_ENABLE		= 0x0100,
	BG1_ENABLE		= 0x0200,
	BG2_ENABLE		= 0x0400,
	BG3_ENABLE		= 0x0800,
	OBJ_ENABLE		= 0x1000,
	WIN0_ENABLE		= 0x2000,
	WIN1_ENABLE		= 0x4000,
	WIN_OBJ			= 0x8000,
};

enum DISPSTAT
{
	VBLANK_STATUS	= 0x01,
	HBLANK_STATUS	= 0x02,
	VCOUNT_EVAL		= 0x04,
	IVBLANK_ENABLE	= 0x08,
	IHBLANK_ENABLE	= 0x10,
	IVCOUNT_ENABLE	= 0x20,
};

enum BGCNT
{
	PRIORITY_0			= 0x0000,
	PRIORITY_1			= 0x0001,
	PRIORITY_2			= 0x0002,
	PRIORITY_3			= 0x0003,
	CHAR_BASE_0000		= 0x0000,
	CHAR_BASE_4000		= 0x0004,
	CHAR_BASE_8000		= 0x0008,
	CHAR_BASE_C000		= 0x000C,
	MOZAIC				= 0x0040,
	COLOR_16			= 0x0000,
	COLOR_256			= 0x0080,
	SCREEN_BASE_0000	= 0x0000,
	SCREEN_BASE_0800	= 0x0100,
	SCREEN_BASE_1000	= 0x0200,
	SCREEN_BASE_1800	= 0x0300,
	SCREEN_BASE_2000	= 0x0400,
	SCREEN_BASE_2800	= 0x0500,
	SCREEN_BASE_3000	= 0x0600,
	SCREEN_BASE_3800	= 0x0700,
	SCREEN_BASE_4000	= 0x0800,
	SCREEN_BASE_4800	= 0x0900,
	SCREEN_BASE_5000	= 0x0A00,
	SCREEN_BASE_5800	= 0x0B00,
	SCREEN_BASE_6000	= 0x0C00,
	SCREEN_BASE_6800	= 0x0D00,
	SCREEN_BASE_7000	= 0x0E00,
	SCREEN_BASE_7800	= 0x0F00,
	SCREEN_BASE_8000	= 0x1000,
	SCREEN_BASE_8800	= 0x1100,
	SCREEN_BASE_9000	= 0x1200,
	SCREEN_BASE_9800	= 0x1300,
	SCREEN_BASE_A000	= 0x1400,
	SCREEN_BASE_A800	= 0x1500,
	SCREEN_BASE_B000	= 0x1600,
	SCREEN_BASE_B800	= 0x1700,
	SCREEN_BASE_C000	= 0x1800,
	SCREEN_BASE_C800	= 0x1900,
	SCREEN_BASE_D000	= 0x1A00,
	SCREEN_BASE_D800	= 0x1B00,
	SCREEN_BASE_E000	= 0x1C00,
	SCREEN_BASE_E800	= 0x1D00,
	SCREEN_BASE_F000	= 0x1E00,
	SCREEN_BASE_F800	= 0x1F00,
	OVERFLOW			= 0x2000,
	SIZE_X256_Y256		= 0x0000,
	SIZE_X512_Y256		= 0x4000,
	SIZE_X256_Y512		= 0x8000,
	SIZE_X512_Y512		= 0xC000,
};

enum BLDMOD
{
	TOP_BG0		= 0x0001,
	TOP_BG1		= 0x0002,
	TOP_BG2		= 0x0004,
	TOP_BG3		= 0x0008,
	TOP_OBJ		= 0x0010,
	TOP_BD		= 0x0020,
	BOT_BG0		= 0x0100,
	BOT_BG1		= 0x0200,
	BOT_BG2		= 0x0400,
	BOT_BG3		= 0x0800,
	BOT_OBJ		= 0x1000,
	BOT_BD		= 0x2000,
	FX_ALPHA	= 0x0040,
	FX_WHITE	= 0x0080,
	FX_BLACK	= 0x00C0,
};

enum DMACNT
{
	DMA_DAC_INCREMENT	= 0x00000000,
	DMA_DAC_DECREMENT	= 0x00200000,
	DMA_DAC_FIXED		= 0x00400000,
	DMA_DAC_RELOAD		= 0x00600000,
	DMA_SAC_INCREMENT	= 0x00000000,
	DMA_SAC_DECREMENT	= 0x00800000,
	DMA_SAC_FIXED		= 0x01000000,
	DMA_SAC_RELOAD		= 0x01800000,
	DMA_REPEAT			= 0x02000000,
	DMA_16				= 0x00000000,
	DMA_32				= 0x04000000,
	DMA_GAMEPAK			= 0x08000000,
	DMA_IMMEDIATE		= 0x00000000,
	DMA_VBLANK			= 0x10000000,
	DMA_HBLANK			= 0x20000000,
	DMA_SPECIAL			= 0x30000000,
	DMA_IRQ				= 0x40000000,
	DMA_ENABLE			= 0x80000000,
};

enum SOUNDCNT_H
{
	SND_VOL_AB_25	= 0x0000,
	SND_VOL_AB_50	= 0x0001,
	SND_VOL_AB_100	= 0x0002,
	SND_VOL_A_50	= 0x0000,
	SND_VOL_A_100	= 0x0004,
	SND_VOL_B_50	= 0x0000,
	SND_VOL_B_100	= 0x0008,
	SND_A_RIGHT		= 0x0100,
	SND_A_LEFT		= 0x0200,
	SND_A_TIMER0	= 0x0000,
	SND_A_TIMER1	= 0x0400,
	SND_A_RESET		= 0x0800,
	SND_B_RIGHT		= 0x1000,
	SND_B_LEFT		= 0x2000,
	SND_B_TIMER0	= 0x0000,
	SND_B_TIMER1	= 0x4000,
	SND_B_RESET		= 0x8000,
};

enum SOUNDCNT_X
{
    SND_CHANNEL1_ENABLE     = 0x0001,
    SND_CHANNEL2_ENABLE     = 0x0002,
    SND_CHANNEL3_ENABLE     = 0x0004,
    SND_CHANNEL4_ENABLE     = 0x0008,
    SND_MASTER_ENABLE		= 0x0080,
};

enum TM_CNT
{
	TM_16MHZ	= 0x0000,
	TM_256KHZ	= 0x0001,
	TM_64KHZ	= 0x0002,
	TM_16KHZ	= 0x0003,
	TM_COUNT_UP	= 0x0004,
	TM_IRQ		= 0x0040,
	TM_ENABLE	= 0x0080,
};

enum BUTTONS
{
	BUTTONA			= 0x0001,
	BUTTONB			= 0x0002,
	BUTTONSELECT	= 0x0004,
	BUTTONSTART		= 0x0008,
	BUTTONRIGHT		= 0x0010,
	BUTTONLEFT		= 0x0020,
	BUTTONUP		= 0x0040,
	BUTTONDOWN		= 0x0080,
	BUTTONR			= 0x0100,
	BUTTONL			= 0x0200,
	KEY_IRQ_ENABLE	= 0x4000,
	KEY_IRQ_OR		= 0x0000,
	KEY_IRQ_AND		= 0x8000,
};

enum IRQS
{
	IRQ_VBLANK	= 0x0001,
	IRQ_HBLANK	= 0x0002,
	IRQ_VCOUNT	= 0x0004,
	IRQ_TIMER0	= 0x0008,
	IRQ_TIMER1	= 0x0010,
	IRQ_TIMER2	= 0x0020,
	IRQ_TIMER3	= 0x0040,
	IRQ_SERIAL	= 0x0080,
	IRQ_DMA0	= 0x0100,
	IRQ_DMA1	= 0x0200,
	IRQ_DMA2	= 0x0400,
	IRQ_DMA3	= 0x0800,
	IRQ_KEYPAD	= 0x1000,
	IRQ_GAMEPAK	= 0x2000,
};

struct OAM_OBJECT
{
    u16 attr0;
    u16 attr1;
    u16 attr2;
    s16 fill;
};

struct OAM_AFFINE
{
    u16 fill0[3];
    s16 pa;
    u16 fill1[3];
    s16 pb;
    u16 fill2[3];
    s16 pc;
    u16 fill3[3];
    s16 pd;
};

enum ATTR0
{
	OM_AFFINE			= 0x0100,
	OM_HIDE				= 0x0200,
	OM_AFFINE_DOUBLE	= 0x0300,
	GFX_BLEND			= 0x0400,
	GFX_WIN				= 0x0800,
	MOSAIC				= 0x1000,
	CM_8BPP				= 0x2000,
	SH_WIDE				= 0x4000,
	SH_TALL				= 0x8000,
};

enum ATTR1
{
	HFLIP				= 0x1000,
	VFLIP				= 0x2000,
	SIZE_8				= 0x0000,
	SIZE_16				= 0x4000,
	SIZE_32				= 0x8000,
	SIZE_64				= 0xC000,
};

#define ATTR2_PRIORITY(x) 	(x << 10)
#define ATTR2_PALETTE(x) 	(x << 12)

#define _palette (u16 *)				0x05000000
#define _obj_palette (u16 *)			0x05000200
#define _vram (u16 *)					0x06000000
#define _vram_spr (u16 *)				0x06010000
#define _oam (u16 *)					0x07000000

#define palette(i) 						(_palette + i)
#define obj_palette(i) 					(_obj_palette + i)
#define vram(offset) 					(_vram + (offset)/sizeof(u16))
#define vram_spr(offset) 				(_vram_spr + (offset)/sizeof(u16))
#define oam 							((OAM_OBJECT*)_oam)

#define rgb(r, g, b) (((b) << 10) | ((g) << 5) | (r))

#endif // GBA_H defined


