#include "gba.h"
#include "main.h"
#include "graphic.h"
#include "irq.h"
#include "ninja.hpp"

void wait_vblank()
{
	// notify that a frame passed
	do_frame = true;
	// count up
	++frame_counter;
	// fetch input
	u16 old_input = hold_input;
	hold_input = ~*REG_KEYINPUT;
	frame_input = hold_input & ~old_input;
	release_input = ~hold_input & old_input;

	*REG_IF = *REG_IF;
}

void wait_vblank_and_rotate_bg()
{
	// copy the OAM buffer
	*REG_DMA3SAD = (u16*)oam_buffer;
	*REG_DMA3DAD = _oam;
	*REG_DMA3CNT = DMA_ENABLE | DMA_IMMEDIATE | DMA_32 | sizeof(oam_buffer)/sizeof(u32);

	s32 scroll_x = game.scroll_x;
	s32 scroll_y = game.scroll_y;

	*REG_BG2HOFS = scroll_x;
	*REG_BG2VOFS = scroll_y;

	scroll_x = scroll_x/8;
	scroll_y = scroll_y/8;

	s32 right = scroll_x + 31;
	s32 old_right = old_scroll_x + 31;

	s32 bot = scroll_y + 21;
	s32 old_bot = old_scroll_y + 21;

	s32 l;
	s32 r;

	s32 t;
	s32 b;

	if (scroll_x < old_scroll_x)
	{
		l = scroll_x;
		r = right < old_scroll_x ? right : old_scroll_x;
	}
	else
	{
		l = old_right > scroll_x ? old_right : scroll_x;
		r = right;
	}

	for (s32 x = l; x < r; ++x)
	{
		for (s32 y = scroll_y; y < bot; ++y)
		{
			*(vram(0xD000) + 32*(y % 32) + (x % 32)) = game.level.r_tiles[x + game.level.m_width*y].tile;
		}
	}

	if (scroll_y < old_scroll_y)
	{
		t = scroll_y;
		b = bot < old_scroll_y ? bot : old_scroll_y;
	}
	else
	{
		t = old_bot > scroll_y ? old_bot : scroll_y;
		b = bot;
	}

	for (s32 x = scroll_x; x < right; ++x)
	{
		for (s32 y = t; y < b; ++y)
		{
			*(vram(0xD000) + 32*(y % 32) + (x % 32)) = game.level.r_tiles[x + game.level.m_width*y].tile;
		}
	}

	old_scroll_x = scroll_x;
	old_scroll_y = scroll_y;

	// notify that a frame passed
	do_frame = true;
	// count up
	++frame_counter;
	// fetch input
	u16 old_input = hold_input;
	hold_input = ~*REG_KEYINPUT;
	frame_input = hold_input & ~old_input;
	release_input = ~hold_input & old_input;


	*REG_IF = *REG_IF;
}
