#include "level.hpp"
#include "ninja.hpp"

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

Level::Level()
{
#ifdef SENTINELS
	__instanceBegin[0] = 'L';
	__instanceBegin[1] = 'v';
	__instanceBegin[2] = 'l';
	__instanceBegin[3] = '{';
	__instanceEnd[0] = '}';
	__instanceEnd[1] = 'L';
	__instanceEnd[2] = 'v';
	__instanceEnd[3] = 'l';
#endif // SENTINEL
	r_tiles = 0;
	m_width = 0;
	m_height = 0;
	m_bumperCount = 0;
}

Level::~Level()
{
}

void Level::load(
		const Tile * tiles, u32 widthTiles, u32 heightTiles,
		const RawVector2i * bumpers, u32 bumperCount)
{
	m_width = widthTiles;
	m_height = heightTiles;
	r_tiles = tiles;
	r_bumpers = bumpers;
	m_bumperCount = bumperCount;
}

bool Level::pixelCollision(s32 x, s32 y, u8 mask)
{
	const Tile & t = tile(pixelsToTiles(x), pixelsToTiles(y));
	return (t.type & mask) || isBumperAtPixels(x, y);
}

bool Level::pixelCollision(const Rect & r, u8 mask, Vector2i * pos)
{
	Vector2i minTilePos(
		pixelsToTiles(r.minX()),
		pixelsToTiles(r.minY()));

	Vector2i maxTilePos(
		pixelsToTiles(r.maxX()),
		pixelsToTiles(r.maxY()));

	Vector2i tilePos;

	// For each tile intersecting the hitbox
	for(tilePos.y = minTilePos.y; tilePos.y <= maxTilePos.y; ++tilePos.y)
	for(tilePos.x = minTilePos.x; tilePos.x <= maxTilePos.x; ++tilePos.x)
	{
		// If the tile is matching
		if(game.level.tile(tilePos).type & mask)
		{
			if(pos)
				*pos = tilePos;
			return true; // Collision
		}
	}

	// Test bumpers
	if(m_bumperCount && isBumperAtPixels(r))
		return true;

	return false; // No collision
}

bool Level::isBumperAtPixels(s32 x, s32 y)
{
	Rect bumperHitBox(0, 0, BUMPER_WIDTH_TILES*TILE_SIZE_PX, BUMPER_HEIGHT_TILES*TILE_SIZE_PX);
	for(u32 i = 0; i < m_bumperCount; ++i)
	{
		bumperHitBox.pos.x = r_bumpers[i].x;
		bumperHitBox.pos.y = r_bumpers[i].y + 8;
		if(bumperHitBox.contains(x, y))
			return true;
	}
	return false;
}

bool Level::isBumperAtPixels(const Rect & r, u32 * index)
{
	Rect bumperHitBox(0, 0, BUMPER_WIDTH_TILES*TILE_SIZE_PX, BUMPER_HEIGHT_TILES*TILE_SIZE_PX);
	for(u32 i = 0; i < m_bumperCount; ++i)
	{
		bumperHitBox.pos.x = r_bumpers[i].x;
		bumperHitBox.pos.y = r_bumpers[i].y + 8;
		if(bumperHitBox.intersects(r))
		{
			if(index != 0)
				*index = i;
			return true;
		}
	}
	return false;
}




