#include "gba.h"
#include "irq.h"
#include "menu.h"
#include "graphic.h"
#include "fx.h"
#include "ninja.hpp"
#include "sound.h"

volatile bool do_frame;
u8 frame_counter;
u16 hold_input;
u16 frame_input;
u16 release_input;

#include "../graphics/credits.h"
#include "../sound/master_loop.h"

int main()
{
	*REG_IME = 1;

	*REG_IRQPTR = (void (*)())&wait_vblank;
	*REG_DISPSTAT = IVBLANK_ENABLE;
	*REG_IE = IRQ_VBLANK;

	// copy tilesets
	*REG_DISPCNT = BG2_ENABLE | MODE_3 | OBJ_ENABLE | OBJ_MAP_1D | FORCE_VBLANK;
	*REG_DMA3SAD = creditsBitmap;
	*REG_DMA3DAD = _vram;
	*REG_DMA3CNT = DMA_ENABLE | DMA_IMMEDIATE | DMA_32 | sizeof(creditsBitmap)/sizeof(u32);
	*REG_DISPCNT = BG2_ENABLE | MODE_3 | OBJ_ENABLE | OBJ_MAP_1D;

	unfade(4);

	u16 wait = 0;
	while(1)
	{
		do_frame = false;
		asm("swi 0x0002");

		if (do_frame)
		{
			++wait;
			if (wait == 240)
			{
				break;
			}
		}
	}

	fade(4);

	enable_sound();
	start_sound_A(master_loop, true);

	while (1)
	{
		frame_counter = 0;

		*REG_IRQPTR = (void (*)())&wait_vblank;
		*REG_DISPSTAT = IVBLANK_ENABLE;
		*REG_IE = IRQ_VBLANK;

		init_menu();

		unfade(4);

		while (1)
		{
			do_frame = false;
			asm("swi 0x0002");

			if (frame_input & BUTTONSTART)
			{
				break;
			}

			if (do_frame)
			{
				update_sound();
				update_menu();
			}
		}

		cleanup_menu();

		fade(12);
		init_game_graphics();
		game.init();
		*REG_DISPCNT &= ~FORCE_VBLANK;
		unfade(12);

		//*REG_IRQPTR = (void (*)())&wait_vblank_and_rotate_bg;

		while (1)
		{
			// enter low power mode until an interrupt arises
			do_frame = false;
			asm("swi 0x0002");
			//asm("swi 0x00050000" ::: "r0", "r1");

			if (frame_input & BUTTONSTART)
			{
				fade(16);
				break;
			}

			// if we are in vblank, draw and compute next frame
			if (do_frame)
			{
				update_sound();
				draw();
				game.update();
				prepare_draw();
			}
		}
	}

	return 0;
}





