#include "main.h"
#include "graphic.h"
#include "fx.h"
#include "ninja.hpp"
#include "sound.h"
#include "../levels/level00.h"
#include "../levels/level01.h"
#include "../levels/level02.h"
#include "../levels/LevelTest.h"

// Construct the global
NinjaGame game;

NinjaGame::NinjaGame()
{
#ifdef SENTINELS
	__instanceBegin[0] = 'N';
	__instanceBegin[1] = 'i';
	__instanceBegin[2] = 'n';
	__instanceBegin[3] = '{';
	__instanceEnd[0] = '}';
	__instanceEnd[1] = 'N';
	__instanceEnd[2] = 'i';
	__instanceEnd[3] = 'n';
#endif
}

NinjaGame::~NinjaGame()
{
}

void NinjaGame::init()
{
	framesBeforeRespawn = 0;
	newGame();
}

void NinjaGame::loadLevel(u32 i)
{
	current_level = i;
	switch(i)
	{
		case 1:
			level.load(level01, LEVEL01_WIDTH, LEVEL01_HEIGHT, level01_bumpers, LEVEL01_BUMPERS);
			level.spawn.set(LEVEL01_SPAWN_X, LEVEL01_SPAWN_Y);
		break;

		case 2:
			level.load(level02, LEVEL02_WIDTH, LEVEL02_HEIGHT, level02_bumpers, LEVEL02_BUMPERS);
			level.spawn.set(LEVEL02_SPAWN_X, LEVEL02_SPAWN_Y);
		break;

		case 0:
		default:
			level.load(level00, LEVEL00_WIDTH, LEVEL00_HEIGHT, level00_bumpers, LEVEL00_BUMPERS);
			level.spawn.set(LEVEL00_SPAWN_X, LEVEL00_SPAWN_Y);
//			level.load(LevelTest, LEVELTEST_WIDTH, LEVELTEST_HEIGHT, LevelTest_bumpers, LEVELTEST_BUMPERS);
//			level.spawn.set(LEVELTEST_SPAWN_X, LEVELTEST_SPAWN_Y);
		break;
	}

	lastCheckPoint = level.spawn;
	respawn();

	update_scroll();
	flush_graphics();
}

void NinjaGame::newGame()
{
	loadLevel(0);
}

void NinjaGame::respawn()
{
	current_avatar = 0;

	avatars[0].init(0, 29, 32);
	avatars[1].init(1, 21, 24);
	avatars[2].init(2, 13, 16);
	avatars[3].init(3, 7, 8);

	//avatars[0].setPositionTiles(level.spawn);
	avatars[0].setPositionTiles(lastCheckPoint);
	avatars[0].active = true;
}

void NinjaGame::update()
{
	if(framesBeforeRespawn > 1)
	{
		--framesBeforeRespawn;
		if(framesBeforeRespawn == 1)
		{
			respawn();
			fade();
		}
		return;
	}
	else if(framesBeforeRespawn == 1)
	{
		framesBeforeRespawn = 0;
		unfade();
		return;
	}

	if (!(frame_input & BUTTONR) || !(frame_input & BUTTONL))
	{
		// pop new avatar
		if (current_avatar < 3)
		{
			Avatar & avatarbis = avatars[current_avatar + 1];
			if (frame_input & BUTTONR)
			{
				avatarbis.box.pos = avatar().box.pos;
				avatarbis.box.pos.x += 4;
				avatarbis.box.pos.y -= avatarbis.box.height;

				if(!game.pixelCollision(avatarbis.box, avatars + current_avatar, avatarbis.collisionMask))
				{
					avatarbis.active = true;
					split_avatar(current_avatar);
					++current_avatar;
					if(level.pixelCollision(avatar().box, Tile::LETHAL))
						avatar().kill();
					else
					{
						avatar().jumpIndex = avatar().popJumpStart();
						sfxSplit();
					}
				}
				else
					sfxError();
			}
		}
		if (current_avatar > 0 && avatar().jumpIndex == 0)
		{
			Avatar & avatarbis = avatars[current_avatar - 1];
			if (frame_input & BUTTONL)
			{
				avatarbis.active = false;
				Rect oldBox = avatarbis.box;
				avatarbis.box.pos = avatar().box.pos;
				avatarbis.box.pos.x -= 4;
				avatarbis.box.pos.y -= avatarbis.box.height - avatar().box.height;

				if(!game.pixelCollision(avatarbis.box, avatars + current_avatar, avatarbis.collisionMask))
				{
					avatar().active = false;
					--current_avatar;
					sfxUnsplit();
				}
				else
				{
					avatarbis.box = oldBox;
					sfxError();
				}
				avatarbis.active = true;
			}
		}
	}

	avatars[3].update();
	avatars[2].update();
	avatars[1].update();
	avatars[0].update();

	if (avatar().box.pos.x > tilesToPixels(level.width()))
	{
		fade();
		loadLevel(++current_level);
		unfade();
	}

	update_scroll();
}

void NinjaGame::update_scroll()
{
	scroll_x = avatar().box.pos.x + (avatar().box.width - RESOLUTION_X)/2;

	if (scroll_x < 0)
	{
		scroll_x = 0;
	}
	else if (scroll_x > (s32)(TILE_SIZE_PX*level.width() - RESOLUTION_X))
	{
		scroll_x = TILE_SIZE_PX*level.width() - RESOLUTION_X;
	}

	scroll_y = avatar().box.pos.y + avatar().box.height - RESOLUTION_Y/2;

	if (scroll_y < 0)
	{
		scroll_y = 0;
	}
	else if (scroll_y > (s32)(TILE_SIZE_PX*level.height() - RESOLUTION_Y))
	{
		scroll_y = TILE_SIZE_PX*level.height() - RESOLUTION_Y;
	}
}

bool NinjaGame::pixelCollision(const Rect & r, const void * except, u8 collisionMask)
{
	if(level.pixelCollision(r, collisionMask))
		return true;
	for(u32 i = 0; i < 4; ++i)
	{
		if((avatars + i) != except && avatars[i].active && avatars[i].box.intersects(r))
			return true;
	}
	return false;
}


