#ifndef NINJA_HPP_INCLUDED
#define NINJA_HPP_INCLUDED

#include "gba.h"
#include "util.hpp"
#include "level.hpp"
#include "avatar.hpp"

// Main class of the game
class NinjaGame
{
#ifdef SENTINELS
private: char __instanceBegin[4];
#endif

public:

	NinjaGame();
	~NinjaGame();

	// Initializes the game at its initial state
	void init();

	// Loads the level and spawns the player in it
	void newGame();

	void loadLevel(u32 i);

	// Teleports the player to the last checkpoint
	void respawn();

	// Updates the game logic, whatever it is
	void update();
	void update_scroll();

	// Upper-left corner of the camera
	s32 scroll_y;
	s32 scroll_x;

	u8 current_avatar;
	Avatar avatars[4];

	inline Avatar & avatar()
	{
		return avatars[current_avatar];
	}

	// Checks if something collides with the given rect
	// (Tests both the map and the objects)
	bool pixelCollision(const Rect & r, const void * except, u8 collisionMask=Tile::BLOCK);

	Level level;
	u8 current_level;

	// Position in tiles of the last taken checkpoint
	Vector2i lastCheckPoint;
	u32 framesBeforeRespawn;

#ifdef SENTINELS
private: char __instanceEnd[4];
#endif
};

// Global access to the game
extern NinjaGame game;

#endif // NINJA_HPP_INCLUDED


