#include "gba.h"
#include "sound.h"
#include "main.h"
#include "../sound/die.h"
#include "../sound/die1.h"
#include "../sound/die2.h"
#include "../sound/group.h"
#include "../sound/group1.h"
#include "../sound/group2.h"
#include "../sound/split.h"
#include "../sound/split1.h"
#include "../sound/split2.h"
#include "../sound/jump.h"
#include "../sound/jump1.h"
#include "../sound/jump2.h"
#include "../sound/bump.h"
#include "../sound/bump1.h"
#include "../sound/bump2.h"
#include "../sound/jumpsmall.h"
#include "../sound/jumpsmall1.h"
#include "../sound/jumpsmall2.h"
#include "../sound/jumpbig.h"
#include "../sound/jumpbig1.h"
#include "../sound/jumpbig2.h"

u8 sfxRand3 = 0;

void _randomize()
{
	sfxRand3 = (sfxRand3+frame_counter) % 3;
}

void sfxJump(u8 size)
{
	if(sound_B != 0)
		return;
	_randomize();
	switch(size)
	{
	case 0:
		switch(sfxRand3)
		{
		case 0: start_sound_B(jumpbig, false); break;
		case 1: start_sound_B(jumpbig1, false); break;
		case 2: start_sound_B(jumpbig2, false); break;
		default: break;
		}
		break;
	case 1:
		switch(sfxRand3)
		{
		case 0: start_sound_B(jump, false); break;
		case 1: start_sound_B(jump1, false); break;
		case 2: start_sound_B(jump2, false); break;
		default: break;
		}
		break;
	case 2:
		switch(sfxRand3)
		{
		case 0: start_sound_B(jumpsmall, false); break;
		case 1: start_sound_B(jumpsmall1, false); break;
		case 2: start_sound_B(jumpsmall2, false); break;
		default: break;
		}
		break;
	default: break;
	}
}

void sfxDie()
{
	if(sound_B != 0)
		return;
	_randomize();
	switch(sfxRand3)
	{
	case 0: start_sound_B(die, false); break;
	case 1: start_sound_B(die1, false); break;
	case 2: start_sound_B(die2, false); break;
	default: break;
	}
}

void sfxBump()
{
	if(sound_B != 0)
		return;
	_randomize();
	switch(sfxRand3)
	{
	case 0: start_sound_B(bump, false); break;
	case 1: start_sound_B(bump1, false); break;
	case 2: start_sound_B(bump2, false); break;
	default: break;
	}
}

void sfxSplit()
{
	if(sound_B != 0)
		return;
	_randomize();
	switch(sfxRand3)
	{
	case 0: start_sound_B(split, false); break;
	case 1: start_sound_B(split1, false); break;
	case 2: start_sound_B(split2, false); break;
	default: break;
	}
}

void sfxUnsplit()
{
	if(sound_B != 0)
		return;
	_randomize();
	switch(sfxRand3)
	{
	case 0: start_sound_B(group, false); break;
	case 1: start_sound_B(group1, false); break;
	case 2: start_sound_B(group2, false); break;
	default: break;
	}
}

void sfxError()
{
	// sweepshift, incdec, time
	*REG_SOUND1CNT_L = (0x2) | (0 << 0x3) | (0x5 << 0x4); // 0x16; // 0b0010110

	// sndlen, duty, steptime, env, initialenv
	//duty=50%,envelope decrement
	*REG_SOUND1CNT_H = (0) | (0x2 << 0x6) | (0x1 << 0x8) | (0 << 0xB) | (0xf << 0xC);

	*REG_SOUND1CNT_X = 0x8400; //frequency=0x0400, loop mode
}

void enable_sound()
{
	*REG_SOUNDCNT_X = SND_MASTER_ENABLE | SND_CHANNEL1_ENABLE;
	*REG_SOUNDCNT_H = SND_VOL_AB_100 | SND_VOL_A_100 | SND_VOL_B_50 | SND_B_TIMER1;

	//full volume, enable sound 1 to left and right
	*REG_SOUNDCNT_L=0x1177;
}


const u32 * sound_A = 0;
u32 length_A;
u32 elapsed_A;
bool repeat_A;

void _start_sound_A(const u32* sound, u32 length, bool repeat)
{
	*REG_SOUNDCNT_H |= SND_A_RESET | SND_A_LEFT | SND_A_RIGHT;

	*REG_DMA1CNT = 0;
	*REG_DMA1SAD = sound + 1;
	*REG_DMA1DAD = REG_SND_FIFO_A;
	*REG_DMA1CNT = DMA_ENABLE | DMA_SPECIAL | DMA_REPEAT | DMA_DAC_FIXED | DMA_32 | 1;

	*REG_SND_FIFO_A = *sound;

	// sampling timer
	*(s16*)REG_TIMER0 = -512;
	*REG_TM0_CNT = TM_16MHZ | TM_ENABLE;

	sound_A = sound;
	length_A = length;
	elapsed_A = 0;
	repeat_A = repeat;
}

void restart_sound_A()
{
	_start_sound_A(sound_A, length_A, repeat_A);
}

void stop_sound_A()
{
	*REG_SOUNDCNT_H |= SND_A_RESET;
	*REG_TM0_CNT = 0;
	sound_A = 0;
}


const u32 * sound_B = 0;
u32 length_B;
u32 elapsed_B;
bool repeat_B;

void _start_sound_B(const u32* sound, u32 length, bool repeat)
{
	*REG_SOUNDCNT_H |= SND_B_RESET | SND_B_LEFT | SND_B_RIGHT;

	*REG_DMA2CNT = 0;
	*REG_DMA2SAD = sound + 1;
	*REG_DMA2DAD = REG_SND_FIFO_B;
	*REG_DMA2CNT = DMA_ENABLE | DMA_SPECIAL | DMA_REPEAT | DMA_DAC_FIXED | DMA_32 | 1;

	*REG_SND_FIFO_B = *sound;

	// sampling timer
	*(s16*)REG_TIMER1 = -512;
	*REG_TM1_CNT = TM_16MHZ | TM_ENABLE;

	sound_B = sound;
	length_B = length;
	elapsed_B = 0;
	repeat_B = repeat;
}

void restart_sound_B()
{
	_start_sound_B(sound_B, length_B, repeat_B);
}

void stop_sound_B()
{
	*REG_SOUNDCNT_H |= SND_B_RESET;
	*REG_TM1_CNT = 0;
	sound_B = 0;
}

void update_sound()
{
	if (sound_A)
	{
		elapsed_A += 32768;
		if (elapsed_A >= length_A*60)
		{
			if (repeat_A)
			{
				restart_sound_A();
			}
			else
			{
				stop_sound_A();
			}
		}
	}

	if (sound_B)
	{
		elapsed_B += 32768;
		if (elapsed_B >= length_B*60)
		{
			if (repeat_B)
			{
				restart_sound_B();
			}
			else
			{
				stop_sound_B();
			}
		}
	}
}
