#ifndef UTIL_HPP_INCLUDED
#define UTIL_HPP_INCLUDED

#include "gba.h"

struct RawVector2i
{
	s32 x;
	s32 y;
};

struct Vector2i
{
	s32 x;
	s32 y;

	Vector2i() : x(0), y(0) {}
	Vector2i(s32 xi, s32 yi) : x(xi), y(yi) {}
	Vector2i(const RawVector2i & rv) : x(rv.x), y(rv.y) {}

	void operator=(const RawVector2i & rv)
	{
		x = rv.x;
		y = rv.y;
	}

	inline void set(s32 newx, s32 newy)
	{
		x = newx;
		y = newy;
	}

	void operator+=(const Vector2i & other)
	{
		x += other.x;
		y += other.y;
	}

};

// A basic AABB in 2D
struct Rect
{
	Vector2i pos; // Upper-left corner
	s32 width;
	s32 height;

	Rect() : width(0), height(0) {}
	Rect(s32 x, s32 y, u32 w, u32 h) : pos(x, y), width(w), height(h) {}

	void setSize(s32 newWidth, s32 newHeight)
	{
		width = newWidth < 0 ? 0 : newWidth;
		height = newHeight < 0 ? 0 : newHeight;
	}

	inline s32 minX() const { return pos.x; }
	inline s32 minY() const { return pos.y; }
	inline s32 maxX() const { return pos.x + width - 1; }
	inline s32 maxY() const { return pos.y + height - 1; }

	bool contains(s32 x, s32 y)
	{
		return x >= minX() && x <= maxX()
			&& y >= minY() && y <= maxY();
	}

	bool intersects(const Rect & other)
	{
		if(maxX() < other.minX())
			return false;
		if(maxY() < other.minY())
			return false;
		if(minX() > other.maxX())
			return false;
		if(minY() > other.maxY())
			return false;
		return true;
	}

	void add(const Vector2i & vec)
	{
		if(vec.x > 0)
			width += vec.x;
		else if(vec.x < 0)
			pos.x += vec.x;

		if(vec.y > 0)
			height += vec.y;
		else if(vec.y < 0)
			pos.y += vec.y;
	}

};

#endif // UTIL_HPP_INCLUDED





